""" Context managers for code generation. """

# Implement a context manager that allows to temporarily define tags globally.
_cache_context_stack = []


class _CacheContext(object):
    def __init__(self, tags, delete=False):
        self.tags = tags
        self.delete = delete

    def __enter__(self):
        _cache_context_stack.append(self.tags)

    def __exit__(self, exc_type, exc_value, traceback):
        _cache_context_stack.pop()

        if self.delete:
            from dune.perftool.generation.cache import delete_cache_items
            delete_cache_items(condition=" and ".join(self.tags))


def cache_context(*tags, **kwargs):
    return _CacheContext(tags, **kwargs)


def get_context_tags():
    result = tuple()
    for items in _cache_context_stack:
        result = result + items
    return result
